#!/bin/bash

#######################################################################
####                                                               ####
####                    processing variables                       ####
####                         CORDEX format                         #### 
####              daily statistics to monthly means                ####  
####                           version 1                           ####  
#######################################################################

set -e

# --------------------
# ---   VARIABLES  ---
# --------------------

# --- CORE: Monthly and seasonal ---
 # var_in=(tas tasmax tasmin pr ps psl huss sfcWind sfcWindmax clt sund rsds rlds hfls hfss rsus rlus evspsbl \
 #         mrfso mrro mrso snw snm rlut rsdt rsut uas vas ua850 va850 ta850 hus850 ua500 va500 zg500 ta500 ua200 va200 ta200 zg200 snc snd sic)  # 43

  var_in=(tas pr psl rsds ta850)  # example only


# --------------------
# --- SIMULATIONS  ---
# --------------------

 # --- EUROPE 44 (CORDEX) EXAMPLES ONLY !!!! ---
  run_in=(201116) # ERA-Interim      EUR-44  evaluation              
  #run_in=(201209) # EC-EARTH        EUR-44  historical        
  #run_in=(201210) # EC-EARTH        EUR-44  rcp45
  #run_in=(201211) # EC-EARTH        EUR-44  rcp85
  

#---- FIRST and LAST YEARS
#fy_in=(1980 1981)   # first year; ERA-Interim 1980-2010
#ly_in=(1980 2010)   #  last year; ERA-Interim 1980-2010

#fy_in=(1951 2001)   # first year; historical 1951-2005
#ly_in=(2000 2005)   #  last year; historical 1951-2005

#fy_in=(2006 2011)   # first year; scenario 2006-2100
#ly_in=(2010 2100)   #  last year; scenario 2006-2100

fy_in=(1980)   #  first year; example 
ly_in=(1980)   #   last year; example


# ---- NUMBER OF YEARS in OUTPUT FILES -----
# if sy=0 all years will be in one output file
#sy_in=(1 10)   # ERA-Interim 1980-2010
#sy_in=(10 5)   # historical 1951-2005  
#sy_in=(5 10)   # scenario 2006-2100

sy_in=(1)


# --- check cordex writer path ---
if [[ -z ${CORDEX_WRITER_PATH} ]]; then
 echo ...  CORDEX_WRITER_PATH not defined: TERMINATED; exit;
else
 echo ... CORDEX_WRITER_PATH ...  $CORDEX_WRITER_PATH
fi 

   
# --- INPUT and OUTPUT PATHs ---
path_in_dm=${CORDEX_WRITER_PATH}'/output_files/day/'
path_out=${CORDEX_WRITER_PATH}'/output_files/mon/'


# --- META INFO ---
 path_meta=${CORDEX_WRITER_PATH}'/writer_meta/' # path to meta files
 cdo='cdo -s'
 freq=mon


# --- Number of variables, runs etc..  ---
 num_var=${#var_in[@]}     
 num_run=${#run_in[@]}     
 num_fy=${#fy_in[@]}       
 num_ly=${#ly_in[@]}       
 num_sy=${#sy_in[@]}

# --- check if fy_in, ly_in and sy_in have the same dimension ---
 if [ $num_fy -ne $num_ly -o $num_fy -ne $num_sy ]; then
  echo ... fy_in, ly_in and sy_in have different dimensions .... TERMINATED
  exit
 fi

# --- check input path ---
 if [ ! -d $path_in_dm ]; then
   echo .... INPUT PATH does not exist, TERMINATED; exit
 fi

# --- check output path ---
 if [ ! -d $path_out ]; then
   echo .... OUTPUT PATH does not exist, TERMINATED; exit
 fi


# ------------------
# --- MAIN BLOCK ---
# ------------------
echo
echo ... START MONTHLY AVERAGING ...
echo
beg_time="$(date +%s)"

# --- SIMULATION LOOP ---  
for ((run=0;run<=num_run-1;run++)); do    # simulation loop

# --- read simulation info ---
 file_run=$path_meta'run_info_cordex.sh'
 . $file_run


# --- VARIABLE LOOP ---
for ((var=0;var<=num_var-1;var++)); do   # variable loop

# --- ALL AVAILABLE DAILY FILES ---
 mask_file_in=$path_in_dm${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_day_'????????'-'????????'.nc'
 flist=`ls $mask_file_in`
 num_file_in=0

 echo
 echo   AVAILABLE DAILY FILES:
 echo

for f in $flist; do
  date_tmp=${f:(-20)}
  fy_tmp=${date_tmp:0:4}
  ly_tmp=${date_tmp:9:4}
   
  fy_file[$num_file_in]=$fy_tmp
  ly_file[$num_file_in]=$ly_tmp
  file_in_all[$num_file_in]=$f
  echo ...  ${file_in_all[$num_file_in]}
  num_file_in=$((num_file_in+1))
done
 echo


# --- YEAR LOOP ---
for ((pp=0;pp<num_fy;pp++)); do

 fy=${fy_in[$pp]} 
 ly=${ly_in[$pp]}
 sy=${sy_in[$pp]}
 year_p=$(($ly - $fy + 1))  # number of years

# --- number of output files ----
if [ $sy -eq 0 ]; then
  sy=$year_p
  file_p=1
else
  file_p=$((year_p/sy))       
  if [ $((year_p%sy)) -ne 0 ]; then
   file_p=$((file_p+1))
  fi
fi

# ----- OUTPUT FILE LOOP -----
for ((ff=1;ff<=file_p;ff++)); do 

 # ---first and last year in output file ---
  fy_out=$((fy+(ff-1)*sy)) # first year in output file
  ly_out=$((fy_out+sy-1))  # last year in output file

 # --- select only given years ---
  f_date=$fy_out'-01-01T00:00:00'
  l_date=$ly_out'-12-31T23:59:00'

  file_out=$path_out${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'$fy_out'-'$ly_out'.nc'
  [ -f $file_out ] && rm $file_out

 # --- post-processing info ---
   echo
   echo
   echo ... VARIABLE NETCDF      ' '... $var_in 
   echo ... SIMULATION    '      '... ${run_in[$run]} '|' $domain '|' $experiment_id '|' $gcm_name '|' $gcm_version_id '|' $rcm_name '|' $rcm_version_id '|'  $fy_out'-'$ly_out '|'
   echo ... CALENDAR   '       ' ... $calendar'   !!!!!!!!!!!'
   echo ... OUTPUT FILE   '    ' ... $file_out
   echo

  for ((nn=0;nn<=$((num_file_in-1));nn++)); do
   if [ ${fy_file[$nn]} -ge $fy_out -a ${fy_file[$nn]} -le $ly_out ] || \
      [ ${ly_file[$nn]} -ge $fy_out -a ${ly_file[$nn]} -le $ly_out ] || \
      [ $fy_out -ge ${fy_file[$nn]} -a $fy_out -le ${ly_file[$nn]} ]; then
      echo ... now processing ... ${file_in_all[$nn]}
      $cdo cat  -monmean -seldate,$f_date,$l_date ${file_in_all[$nn]} $file_out
   fi
  done

 if [ -f "$file_out" ]; then

# ---------------------
# ----     TIME    ----
# ---------------------
  ncrename -d nb2,bnds -h $file_out
  ncatted -a long_name,time,o,c,"time" -a axis,time,o,c,"T" -h $file_out
  ncap2 -h -O -s 'time=( (time_bnds(:,0)+time_bnds(:,1))/2. )' $file_out $file_out

 
# ------------------------------
# --- HORIZONTAL COORDINATES ---
# ------------------------------
  ncrename -d x,rlon -d y,rlat -h $file_out
  ncatted -a _CoordinateAxisType,lon,d,, -a _CoordinateAxisType,lat,d,, -h $file_out
  ncks -A -a -v rotated_pole,rlon,rlat -h ${file_in_all[0] } $file_out


# -----------------------------
# ---  VERTICAL COORDINATE  ---
# -----------------------------
  case ${var_in[$var]} in
    tas | tasmax | tasmin | huss | sfcWind | sfcWindmax | wsgsmax | uas | vas)
       ncks -A -a -v height -h ${file_in_all[0] } $file_out;;
    ua[0-9][0-9][0-9] | va[0-9][0-9][0-9] | zg[0-9][0-9][0-9] | hus[0-9][0-9][0-9] | ta[0-9][0-9][0-9])
       ncks -A -a -v plev -h ${file_in_all[0] } $file_out;;
    cll | clm | clh )
       ncks -A -a -v plev,plev_bnds -h ${file_in_all[0] } $file_out;;
  esac


# ---------------------------
# --- VARIABLE ATTRIBUTES ---
# ---------------------------
  ncatted -a grid_mapping,${var_in[$var]},o,c,"rotated_pole" \
          -a missing_value,${var_in[$var]},o,f,1.e+20 -h $file_out

  case ${var_in[$var]} in
    tasmax | sfcWindmax | wsgsmax)
       ncatted -a cell_methods,${var_in[$var]},o,c,"time: maximum within days time: mean over days" -h $file_out;;
    tasmin)
       ncatted -a cell_methods,${var_in[$var]},o,c,"time: minimum within days time: mean over days" -h $file_out;;
      sund)
       ncatted -a cell_methods,${var_in[$var]},o,c,"time: sum within days time: mean over days" -h $file_out;;
  esac


# -----------------------------------
# ---  DATES in FILE NAME: YYYYMM ---
# -----------------------------------
  num_time=`$cdo ntime $file_out`
  f_date=`$cdo showdate -seltimestep,1 $file_out`; f_date=${f_date// /}; f_date=${f_date//-/}
  l_date=`$cdo showdate -seltimestep,$num_time $file_out`; l_date=${l_date// /}; l_date=${l_date//-/}
  file_cordex=$path_out${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'${f_date:0:6}'-'${l_date:0:6}'.nc'
  mv $file_out $file_cordex


# -------------------------
# --- GLOBAL ATRIBUTES  ---
# -------------------------
  ncatted -a creation_date,global,o,c,"$( date +%Y'-'%m'-'%d'-T'%T'Z')" -h $file_cordex
  ncatted -a frequency,global,o,c,"mon" -h $file_cordex
  ncatted -a history,global,d,, -h $file_cordex
  ncatted -a CDI,global,d,, -h $file_cordex
  ncatted -a CDO,global,d,, -h $file_cordex
  ncatted -a NCO,global,d,, -h $file_cordex

# --- traking_id attribute (recommended) ---
# ncatted -a tracking_id,global,o,c,`uuid -v 4` -h $file_cordex
 
else
    echo ... OUTPUT FILE does not exist $file_out
fi # if file_out exists

   done  # output file loop
  done  # year loop
 done  # variable loop
done  # simulation loop

end_time="$(date +%s)"
echo
echo ELAPSED TIME: "$(expr $end_time - $beg_time)" sec
echo
echo "END"
