#!/bin/bash

#######################################################################
####                                                               ####
####                   processing variables                        ####
####                       CORDEX format                           #### 
####                 monthly to seasoanl means                     ####  
####                         version 1                             ####  
#######################################################################

set -e
           
# --------------------
# ---   VARIABLES  ---
# --------------------

# --- CORE: Monthly and seasonal ---
 # var_in=(tas tasmax tasmin pr ps psl huss sfcWind sfcWindmax clt sund rsds rlds hfls hfss rsus rlus evspsbl \
 #         mrfso mrro mrso snw snm rlut rsdt rsut uas vas ua850 va850 ta850 hus850 ua500 va500 zg500 ta500 ua200 va200 ta200 zg200 snc snd sic)  # 43
 
 var_in=(tas pr psl rsds ta850)  # example only


# --------------------
# --- SIMULATIONS  ---
# --------------------

 # --- EUROPE 44 (CORDEX) EXAMPLES ONLY !!!! ---
  run_in=(201116) # ERA-Interim     EUR-44  evaluation              
  #run_in=(201209) # EC-EARTH        EUR-44  historical        
  #run_in=(201210) # EC-EARTH        EUR-44  rcp45
  #run_in=(201211) # EC-EARTH        EUR-44  rcp85


# --- check cordex writer path ---
if [[ -z ${CORDEX_WRITER_PATH} ]]; then
 echo ...  CORDEX_WRITER_PATH not defined: TERMINATED; exit;
else
 echo ... CORDEX_WRITER_PATH ...  $CORDEX_WRITER_PATH
fi 

   
# --- INPUT and OUTPUT PATHs ---
path_in_dm=${CORDEX_WRITER_PATH}'/output_files/mon/'
path_out=${CORDEX_WRITER_PATH}'/output_files/sem/'


# --- META INFO ---
 path_meta=${CORDEX_WRITER_PATH}'/writer_meta/' # path to meta files
 ref_time='1949-12-01,00:00'  # reference time
 cdo='cdo -s'
 freq=sem

# --- Number of variables, runs etc..  ---
 num_var=${#var_in[@]}      # number of variables 
 num_run=${#run_in[@]}      # number of simulations

# --- check input path ---
 if [ ! -d $path_in_dm ]; then
   echo .... INPUT PATH does not exist, TERMINATED; exit
 fi

# --- check output path ---
 if [ ! -d $path_out ]; then
   echo .... OUTPUT PATH does not exist, TERMINATED; exit
 fi


# ------------------
# --- MAIN BLOCK ---
# ------------------
echo
echo ... START SEASONAL AVERAGING ...
echo
beg_time="$(date +%s)"


# --- SIMULATION LOOP ---  
for ((run=0;run<=num_run-1;run++)); do    # simulation loop

# --- read simulation info ---
 file_run=$path_meta'run_info_cordex.sh'
 . $file_run

# --- VARIABLE LOOP ---
for ((var=0;var<=num_var-1;var++)); do   # variable loop


# --- ALL AVAILABLE MONTHLY FILES ---
 mask_file_in=$path_in_dm${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_mon_'??????'-'??????'.nc'
 flist=`ls $mask_file_in`
 num_file_in=0

echo
echo   AVAILABLE MONTHLY FILES:
echo

for f in $flist; do
   date_tmp=${f:(-16)}
   fy_tmp=${date_tmp:0:4}
   ly_tmp=${date_tmp:7:4}

   num_file_in=$((num_file_in+1))
   fy_file[$num_file_in]=$fy_tmp
   ly_file[$num_file_in]=$ly_tmp
   file_in_all[$num_file_in]=$f
   echo ...  ${file_in_all[$num_file_in]}
done
   echo
   echo


# --- MONTHLY MEAN FILE LOOP ---
for ((ff=beg_file;ff<=num_file_in;ff++)); do 
 file_out=
 file_tmp0=

   if [ $ff -eq  1  ]; then
     echo '   '    ${fy_file[ff]}'-'${ly_file[ff]}
     echo ----------------
     echo ... now processing ... ${file_in_all[$ff]}
     file_out=$path_out${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'${fy_file[ff]}'_'${ly_file[ff]}'.nc'
     file_tmp0=$path_out'tmp0_'${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'${fy_file[ff]}'_'${ly_file[ff]}'.nc'
     [ -f $file_out ] && rm $file_out
     [ -f $file_tmp0 ] && rm $file_tmp0   
     echo ... OUTPUT FILE   '    ' ... $file_out
     $cdo seasmean -seldate,${fy_file[ff]}'-03-01TT00:00:00',${ly_file[ff]}'-11-30TT00:00:00' ${file_in_all[$ff]} $file_tmp0
     $cdo -setreftime,$ref_time -setday,1 -settime,00:00:00 -settunits,days  $file_tmp0 $file_out
     [ -f $file_tmp0 ] && rm $file_tmp0 
   fi

   if [ $ff -gt  1 ]; then
    echo
    echo '   '    ${fy_file[ff]}'-'${ly_file[ff]}
    echo ----------------
    echo ... now processing ... ${file_in_all[$ff]}
    file_out=$path_out${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'${fy_file[ff]}'_'${ly_file[ff]}'.nc'
    file_tmp0=$path_out'tmp0_'${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'${fy_file[ff]}'_'${ly_file[ff]}'.nc'
    [ -f $file_out ] && rm $file_out
    [ -f $file_tmp0 ] && rm $file_tmp0   
    echo ... OUTPUT FILE   '    ' ... $file_out
    $cdo -seldate,${ly_file[$((ff-1))]}'-12-01TT00:00:00',${ly_file[$((ff-1))]}'-12-31TT00:00:00' ${file_in_all[$((ff-1))]} $file_tmp0
    $cdo cat -seldate,${fy_file[ff]}'-01-01TT00:00:00',${ly_file[ff]}'-11-30TT00:00:00' ${file_in_all[$ff]} $file_tmp0
    $cdo -setreftime,$ref_time -setday,1 -settime,00:00:00 -settunits,days -seasmean $file_tmp0 $file_out
    [ -f $file_tmp0 ] && rm $file_tmp0
   fi


if [ -f "$file_out" ]; then


# -----------------------
# --- TIME ATRIBUTES  ---
# -----------------------
  ncatted -a long_name,time,c,c,"time" -a axis,time,c,c,"T" -h $file_out


# ------------------------------
# --- HORIZONTAL COORDINATES ---
# ------------------------------
  ncrename -d x,rlon -d y,rlat -h $file_out
  ncatted -a _CoordinateAxisType,lon,d,, -a _CoordinateAxisType,lat,d,, -h $file_out
  ncks -A -a -v rotated_pole,rlon,rlat -h ${file_in_all[$((ff))] } $file_out


# -----------------------------
# ---  VERTICAL COORDINATE  ---
# -----------------------------
  case ${var_in[$var]} in
    tas | tasmax | tasmin | huss | sfcWind | sfcWindmax | wsgsmax | uas | vas)
       ncks -A -a -v height -h ${file_in_all[1] } $file_out;;
    ua[0-9][0-9][0-9] | va[0-9][0-9][0-9] | zg[0-9][0-9][0-9] | hus[0-9][0-9][0-9] | ta[0-9][0-9][0-9])
       ncks -A -a -v plev -h ${file_in_all[1] } $file_out;;
    cll | clm | clh )
       ncks -A -a -v plev,plev_bnds -h ${file_in_all[1] } $file_out;;
  esac


# ---------------------------
# --- VARIABLE ATTRIBUTES ---
# ---------------------------
  ncatted -a grid_mapping,${var_in[$var]},o,c,"rotated_pole" \
           -a missing_value,${var_in[$var]},o,f,1.e+20 -h $file_out


# ---------------------
# ---  TIME BOUNDS  ---
# ---------------------
 file_bnds_sem=$path_meta'add_time_bnds_sem_cordex.sh'
 . $file_bnds_sem


# -------------------------------------
# ---  DATES in FILE NAME: YYYYMMDD ---
# -------------------------------------
  num_time=`$cdo ntime $file_out`
  f_date=`$cdo showdate -seltimestep,1 $file_out`; f_date=${f_date// /}; f_date=${f_date//-/}
  f_year=${f_date:0:4}
  f_month=${f_date:4:2}

  l_date=`$cdo showdate -seltimestep,$num_time $file_out`; l_date=${l_date// /}; l_date=${l_date//-/}
  l_year=${l_date:0:4}
  l_month=${l_date:4:2}

 case $f_month in
    04) f_month_out=03;;
    07) f_month_out=06;;
    10) f_month_out=09;;
    01) f_month_out=12;;
 esac

 case $l_month in
    04) l_month_out=05;;
    07) l_month_out=08;;
    10) l_month_out=11;;
    01) l_month_out=2;;
 esac

 if [ $ff -gt  1 ]; then
  f_year=$((f_year-1))
 fi

 file_cordex=$path_out${var_in[$var]}'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'$f_year$f_month_out'-'$l_year$l_month_out'.nc'
 mv $file_out $file_cordex


# -------------------------
# --- GLOBAL ATRIBUTES  ---
# -------------------------
  ncatted -a creation_date,global,o,c,"$( date +%Y'-'%m'-'%d'-T'%T'Z')" -h $file_cordex
  ncatted -a frequency,global,o,c,"sem" -h $file_cordex
  ncatted -a history,global,d,, -h $file_cordex
  ncatted -a CDI,global,d,, -h $file_cordex
  ncatted -a CDO,global,d,, -h $file_cordex
  ncatted -a NCO,global,d,, -h $file_cordex

# --- traking_id attribute (recommended) ---
# ncatted -a tracking_id,global,o,c,`uuid -v 4` -h $file_cordex

 fi # if file_out exists

  done  # output file loop
 done  # variable loop
done  # simulation loop

end_time="$(date +%s)"
echo
echo ELAPSED TIME: "$(expr $end_time - $beg_time)" sec
echo
echo "END"
